package com.mh.user.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.mh.user.entity.DataResultChEntity;
import com.mh.user.entity.DataResultClEntity;
import org.apache.ibatis.annotations.*;

import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project mh_esi
 * @description 数据冷量计
 * @date 2024-07-10 15:20:55
 */
@Mapper
public interface DataResultChMapper extends BaseMapper<DataResultChEntity> {

    //----------------------------------------------------------------------------------------------
    //保存冷量计数据
    @Insert("insert into data_result_ch(device_addr,device_type,fun_code,register_addr,register_name,cur_value,cur_date,project_id,grade) values (" +
            " #{deviceAddr},#{deviceType},#{funCode},#{registerAddr},#{registerName},#{curValue},#{curDate},#{projectId},#{grade})")
    void saveDataResultCh(DataResultChEntity dataResultChEntity);

    @Update("<script>" +
            "  update data_result_ch set  " +
            "  <if test='deviceAddr!=null'>  device_addr = #{deviceAddr} </if>" +
            "  <if test='deviceType!=null'> , device_type = #{deviceType} </if>" +
            "  <if test='funCode!=null'> , fun_code = #{funCode} </if>" +
            "  <if test='registerAddr!=null'> , register_addr = #{registerAddr} </if>" +
            "  <if test='registerName!=null'> , register_name = #{registerName} </if>" +
            "  <if test='curValue!=null'> , cur_value = #{curValue} </if>" +
            "  <if test='projectId!=null'> , project_id = #{projectId} </if>" +
            "  <if test='grade!=null'> , grade = #{grade} </if>" +
            "   where cur_date=#{curDate} and device_addr=#{deviceAddr}  and  device_type=#{deviceType}" +
            "</script>")
    void updateDataResultCh(DataResultChEntity dataResultChEntity);

    @Select("select count(*) from  data_result_ch " +
            " where  cur_date=#{curDate}  " +
            " and  device_addr=#{deviceAddr}  " +
            " and  register_addr=#{registerAddr}  " +
            " and  project_id=#{projectId} " +
            " and grade = #{grade} ")
    int selectDataResultChCount(@Param("curDate") String curDate,
                                @Param("deviceAddr") String deviceAddr,
                                @Param("registerAddr") String registerAddr,
                                @Param("projectId") String projectId,
                                @Param("grade") int grade);

    @Select("select " +
            " t1.project_id, " +
            " sum(cast (t1.curValue AS decimal(18, 2)))as cur_value, " +
            " t1.cur_date, " +
            " t2.project_name " +
            "from " +
            " data_result_ch t1 " +
            "join project_info t2 on " +
            " t1.project_id = t2.id " +
            "where " +
            " t1.project_id = #{projectId} " +
            " and t1.cur_date >= #{startDate} " +
            " and t1.cur_date <= #{curDate} " +
            "group by " +
            " t1.cur_date, " +
            " t1.project_id, " +
            " t2.project_name " +
            "order by " +
            " t1.cur_date ")
    List<DataResultChEntity> queryDataResultCh(@Param("projectId") String projectId, @Param("startDate") String startDate, @Param("curDate") String curDate);

}
