package com.mh.user.service.impl;

import com.mh.user.entity.DataResultClEntity;
import com.mh.user.entity.DataResultEntity;
import com.mh.user.entity.DataResultChEntity;
import com.mh.user.entity.DeviceInstallEntity;
import com.mh.user.mapper.*;
import com.mh.user.service.DataResultService;
import com.mh.user.service.DeviceInstallService;
import com.mh.user.utils.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class DataResultServiceImpl implements DataResultService {

    @Autowired
    DataResultMapper dataResultMapper;

    @Autowired
    DeviceInstallMapper deviceInstallMapper;

    @Autowired
    DeviceInstallService deviceInstallService;

    @Resource
    DataResultChMapper dataResultChMapper;

    @Resource
    DataResultClMapper dataResultClMapper;

    @Resource
    DataResultClBakMapper dataResultClBakMapper;


    @Resource
    DataResultChillerMapper dataResultChillerMapper;

    @Resource
    DataResultOneMiMapper dataResultOneMiMapper;

    @Resource
    DataResultFiveMiMapper dataResultFiveMiMapper;

    @Resource
    DataResultFifteenMiMapper dataResultFifteenMiMapper;

    @Resource
    DataResultNowMapper dataResultNowMapper;

    private final SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Override
    public void saveDataResult(DataResultEntity dataResultEntity) {

        try {
            //获取系统日期
            Date date = new Date();
            if (dataResultEntity.getCurDate() == null) {
                dataResultEntity.setCurDate(date);
            }
            //从安装表获取设备信息
            DeviceInstallEntity deviceInstallEntity = deviceInstallMapper.selectDevice(dataResultEntity.getDeviceAddr(), dataResultEntity.getDeviceType(), dataResultEntity.getProjectId());
            //判断是否有记录
//            int r = dataResultMapper.selectDataResultCount(sdf1.format(dataResultEntity.getCurDate()),
//                    dataResultEntity.getDeviceAddr(),dataResultEntity.getDeviceType());
//            if (r==0){//插入记录
            DataResultEntity data = new DataResultEntity();
            if (dataResultEntity.getDeviceType().equals("水表") || dataResultEntity.getDeviceType().equals("电表")) {
//                    String str=dataResultEntity.getDeviceAddr();
//                    data.setDeviceAddr(str.substring(str.length()-8,str.length())); //通讯地址
                data.setDeviceAddr(dataResultEntity.getDeviceAddr()); //通讯地址
            } else {
                data.setDeviceAddr(dataResultEntity.getDeviceAddr()); //通讯地址
            }
            data.setDeviceType(dataResultEntity.getDeviceType()); //设备类型
            String curDate = sdf1.format(dataResultEntity.getCurDate());
            data.setCurDate(sdf1.parse(curDate));//当前日期
            data.setCurValue(dataResultEntity.getCurValue());     //当前读数
            data.setProjectId(deviceInstallEntity.getProjectId());   //楼栋编号
            data.setRatio(deviceInstallEntity.getRatio());             //倍率
            BigDecimal lastValue = deviceInstallEntity.getLastValue();       //安装基表上次读数
            BigDecimal calcValue = dataResultEntity.getCurValue().subtract(lastValue); //计算用量
            data.setCalcValue(calcValue);//用量
            if (deviceInstallEntity.getLastDate() != null) {//上次抄表日期
                String lastDate = sdf1.format(deviceInstallEntity.getLastDate());
                data.setLastDate(sdf1.parse(lastDate));
            } else {
                String lastDate = sdf1.format(date);
                data.setLastDate(sdf1.parse(lastDate));
            }
            data.setLastValue(lastValue);//上次读数
            data.setGrade(dataResultEntity.getGrade());//级别
            dataResultMapper.saveDataResult(data);         //插入新的记录
//            }else {//修改记录的curValue、calcValue
//                DataResultEntity data2=dataResultMapper.selectDataResult(sdf1.format(dataResultEntity.getCurDate()),dataResultEntity.getDeviceAddr(),dataResultEntity.getDeviceType());
//                double lastValue=data2.getLastValue();       //安装基表上次读数
//                double calcValue=dataResultEntity.getCurValue()-lastValue; //计算用量
//                String curDate=sdf1.format(dataResultEntity.getCurDate());
//                data2.setCurDate(sdf1.parse(curDate));//当前日期
//                data2.setCurValue(dataResultEntity.getCurValue());//当前读数
//                data2.setCalcValue(calcValue);//用量
//
//                dataResultMapper.updateDataResult(data2);  //更新记录
//            }
            //修改安装表中lastValue,lastDate的值
            deviceInstallMapper.updateLastValue(deviceInstallEntity.getId(), dataResultEntity.getCurValue(), date);
        } catch (Exception e) {
            e.printStackTrace();

        }
    }

    @Override
    public void updateDataResult(DataResultEntity dataResultEntity) {
        dataResultMapper.updateDataResult(dataResultEntity);
    }

    @Override
    public List<DataResultEntity> queryDataResult(String projectId, String startDate, String endDate, String deviceType, int page, int limit) {
        return dataResultMapper.queryDataResult(projectId, startDate, endDate, deviceType, page, limit);
    }

    @Override
    public int getDataResultCount(String projectId, String startDate, String endDate, String deviceType, int page, int limit) {
        return dataResultMapper.getDataResultCount(projectId, startDate, endDate, deviceType, page, limit);
    }

    //-------------------------------------------------------------------------------------
    //保存流量计用量
    @Override
    public void saveDataResultCl(DataResultClEntity dataResultClEntity) throws ParseException {
        try {
            //获取系统日期
            Date date = new Date();
            if (dataResultClEntity.getCurDate() == null) {
                dataResultClEntity.setCurDate(date);
            }
            //从安装表获取设备信息
            DeviceInstallEntity deviceInstallEntity = deviceInstallMapper.selectDevice(dataResultClEntity.getDeviceAddr(), dataResultClEntity.getDeviceType(), dataResultClEntity.getProjectId());
            if (null == deviceInstallEntity) {
                return;
            }
            BigDecimal lastValue = deviceInstallEntity.getLastValue();
            BigDecimal curValue = dataResultClEntity.getCurValue();
            dataResultClEntity.setRatio(deviceInstallEntity.getRatio()); //倍率
//            if ((curValue-lastValue)<1000 && (curValue-lastValue)>=0){
            if (curValue.compareTo(lastValue) >= 0) {
                dataResultClMapper.saveDataResultCl(dataResultClEntity);         //插入新的记录
                //修改安装表中lastValue,lastDate的值
                deviceInstallMapper.updateLastValue(deviceInstallEntity.getId(), curValue, date);
            }

        } catch (Exception e) {
            log.error("保存流量计用量失败", e);
        }
    }

    //保存流量计用量
    @Override
    public void saveDataResultCl_bak(DataResultClEntity dataResultClEntity) throws ParseException {
        try {
            //获取系统日期
            if (dataResultClEntity.getCurDate() == null) {
                Date date = new Date();
                dataResultClEntity.setCurDate(date);
            }
            //从安装表获取设备信息
            DeviceInstallEntity deviceInstallEntity = deviceInstallMapper.selectDevice(dataResultClEntity.getDeviceAddr(), dataResultClEntity.getDeviceType(), dataResultClEntity.getProjectId());
            dataResultClEntity.setRatio(deviceInstallEntity.getRatio()); //倍率
            String curDate = DateUtil.dateToString(dataResultClEntity.getCurDate(), "yyyy-MM-dd HH:mm:ss");
            String deviceAddr = deviceInstallEntity.getDeviceAddr();
            String registerAddr = dataResultClEntity.getRegisterAddr();
            String projectId = dataResultClEntity.getProjectId();
            int row = dataResultClBakMapper.selectDataResultChBakCount(curDate, deviceAddr, registerAddr, projectId);//无记录
            if (row == 0) {
                dataResultClBakMapper.saveDataResultCl_bak(dataResultClEntity);         //插入新的记录
            }

        } catch (Exception e) {
            log.error("保存流量计用量异常", e);
        }
    }

    @Override
    public void updateDataResultCl(DataResultClEntity dataResultClEntity) {
        dataResultClMapper.updateDataResultCl(dataResultClEntity);
    }

    //-------------------------------------------------------------------------------------
    @Override
    public void saveDataResultCh(DataResultChEntity dataResultChEntity) throws ParseException {
        try {
            //从安装表获取设备信息
            String deviceAddr = dataResultChEntity.getDeviceAddr();
            String registerAddr = dataResultChEntity.getRegisterAddr();
            String curDate = DateUtil.dateToString(dataResultChEntity.getCurDate(), "yyyy-MM-dd HH:mm:ss");
            String projectId = dataResultChEntity.getProjectId();
            int row = dataResultChMapper.selectDataResultChCount(curDate, deviceAddr, registerAddr, projectId, dataResultChEntity.getGrade());//无记录
            if (row == 0) {
                dataResultChMapper.saveDataResultCh(dataResultChEntity);         //插入新的记录
            }

        } catch (Exception e) {
            log.error("保存冷暖器用量异常", e);
        }
    }

    @Override
    public void updateDataResultCh(DataResultChEntity dataResultChEntity) {
        dataResultChMapper.saveDataResultCh(dataResultChEntity);
    }

    @Override
    public List<DataResultChEntity> queryDataResultMi(String projectId, String startDate, String curDate, int page, int limit, int type) {
        if (type == 0) {
            return dataResultOneMiMapper.queryDataResultOneMi(projectId, startDate, curDate, page, limit);
        } else if (type == 1) {
            return dataResultFiveMiMapper.queryDataResultFiveMi(projectId, startDate, curDate, page, limit);
        } else if (type == 2) {
            return dataResultFifteenMiMapper.queryDataResultFifteenMi(projectId, startDate, curDate, page, limit);
        } else {
            return dataResultOneMiMapper.queryDataResultOneMi(projectId, startDate, curDate, page, limit);
        }
    }

    @Override
    public int dataResultOneMiCount(String projectId, String startDate, String curDate) {
        return dataResultOneMiMapper.dataResultOneMiCount(projectId, startDate, curDate);
    }

    @Override
    public int dataResultFiveMiCount(String projectId, String startDate, String curDate) {
        return dataResultFiveMiMapper.dataResultFiveMiCount(projectId, startDate, curDate);
    }

    @Override
    public int dataResultFifteenMiCount(String projectId, String startDate, String curDate) {
        return dataResultFifteenMiMapper.dataResultFifteenMiCount(projectId, startDate, curDate);
    }

    @Override
    public List<DataResultChEntity> queryDataResultSec(String projectId, String startDate, String curDate) {
        return dataResultChMapper.queryDataResultCh(projectId, startDate, curDate);
    }

    @Override
    public void saveDataResultChiller(DataResultChEntity dataResultChEntity) {

        try {

            String deviceAddr = dataResultChEntity.getDeviceAddr();
            String registerAddr = dataResultChEntity.getRegisterAddr();

            String str = "";
            if (dataResultChEntity.getCurDate() != null) {
                str = sdf1.format(dataResultChEntity.getCurDate());
            } else {
                Date date = new Date();//获取系统日期
                str = sdf1.format(date);
            }

            String curDate = "";
            curDate = str.substring(0, 16) + ":00";
            String projectId = dataResultChEntity.getProjectId();
            dataResultChEntity.setCurDate(sdf1.parse(curDate));
            int row = dataResultChillerMapper.selectDataResultChillerCount(curDate, deviceAddr, registerAddr, projectId, dataResultChEntity.getFunCode());//无记录
            if (row == 0) {
                dataResultChillerMapper.saveDataResultChiller(dataResultChEntity);       //插入新的记录
            } else {
                dataResultChillerMapper.updateDataResultChiller(dataResultChEntity);     //修改
            }
        } catch (Exception e) {
            log.error("保存数据失败", e);
        }
    }

    @Override
    public void updateDataResultChiller(DataResultChEntity dataResultChEntity) {
        dataResultChillerMapper.updateDataResultChiller(dataResultChEntity);
    }

    @Override
    public void saveDataResultNow(DataResultChEntity dataResultChEntity) {
        String deviceAddr = dataResultChEntity.getDeviceAddr();
        String registerAddr = dataResultChEntity.getRegisterAddr();
        String deviceType = dataResultChEntity.getDeviceType();
        String projectId = dataResultChEntity.getProjectId();
        dataResultNowMapper.deleteDataResultNow(deviceAddr, deviceType, registerAddr, projectId);
        dataResultNowMapper.saveDataResultNow(dataResultChEntity);
    }

    @Override
    public void deleteDataResultNow(String deviceAddr, String deviceType, String registerAddr, String projectId) {
        dataResultNowMapper.deleteDataResultNow(deviceAddr, deviceType, registerAddr, projectId);
    }

    @Override
    public List<DataResultChEntity> selectDataResultNow(String deviceAddr, String deviceType, String projectId) {
        return dataResultNowMapper.selectDataResultNow(deviceAddr, deviceType, projectId);
    }

    @Override
    public List<DataResultChEntity> queryDataResultChiller(String projectId, String deviceAddr, String registerName, String startDate, String curDate, int page, int limit) {
        return dataResultChillerMapper.queryDataResultChiller(projectId, deviceAddr, registerName, startDate, curDate, page, limit);
    }

    @Override
    public int dataResultChillerCount(String projectId, String deviceAddr, String registerName, String startDate, String curDate) {
        return dataResultChillerMapper.dataResultChillerCount(projectId, deviceAddr, registerName, startDate, curDate);
    }
}
